#include "cvos.h"
#include "appinfo.h" 
#include "..\resource\ovu.h"

extern STATUS AppMain(U32 cmd, P_U32 param);

// define date struct
typedef struct tagOVUDATE
{
	U16	y;
	U16 m;
	U16 d;
} OVUDATE;

/* Global variables */
U16 ovu_Gperiod;
U8 ovu_Gdatelbl[6][3] = {
	"00", "0", "0",
	"00", "0", "0"
};

U8 ovu_Gdays[] = {31, 0, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};

/* Event Handler */
Boolean ApplicationHandleEvent(EventStruct * e);
Boolean OvuFormHandleEvent(EventStruct * e) ;		// Form is Ovu (1000)

/* useful function */
U16 Ovu_strlen(U8 *str)
{
	U16 ret;
	U8 *p;
	
	if (!str) return 0;
	p = str;
	ret = 0;
	while (*p++) ret++;
	
	return ret;
}

int Ovu_itoa(U16 n, U8 *str, U16 digit)
{
	int i;
	
	if (!str) return 0;
	
	for (i=digit; i>0; i--)
		str[digit-i] = n % (U16)Mpow(10, i) / (U16)Mpow(10, i-1) + '0';
		
	str[digit] = 0;
	
	return 1;
}

U16 Ovu_atoi(U8 *str)
{
	U16 len;
	U16 val;
	U8  c;
	int i;
	
	if (!str) return 0;
	
	len = Ovu_strlen(str);
	val = 0;
	for (i=0; i<len; i++)
	{
		c = str[i];
		if (c == ' ') c = '0';
		if (c >= '0' && c <= '9')
			val += (c - '0') * Mpow(10, len - ((len==1) ? 0 : i) - 1);
		else
			return 0;
	}
	
	return val;
}

U16 Ovu_addday(OVUDATE *psdate, OVUDATE *prdate, int diff)
{
	int rdays;
	rdays = psdate->d + diff;
	if (rdays > ovu_Gdays[psdate->m - 1])
	{
		prdate->d = rdays - ovu_Gdays[psdate->m - 1];
		if (psdate->m == 12)
		{
			prdate->m = 1;
			prdate->y = psdate->y + 1;
		}
		else
		{
			prdate->m = psdate->m + 1;
			prdate->y = psdate->y;
		}
	}
	else if (rdays <= 0)
	{
		if (psdate->m == 1)
		{
			prdate->d = ovu_Gdays[11] + rdays;
			prdate->m = 12;
			prdate->y = psdate->y - 1;
		}
		else
		{
			prdate->d = ovu_Gdays[psdate->m - 2] + rdays;
			prdate->m = psdate->m - 1;
			prdate->y = psdate->y;
		}
	}
	else
	{
		prdate->d = rdays;
		prdate->m = psdate->m;
		prdate->y = psdate->y;
	}
	
	return 1;
}


/*************************************************************************/

void InitVariables()
{
	U16 year, month, day, hour, minute, second;
	
	DateTimeRead(&year, &month, &day, &hour, &minute, &second);

	Ovu_itoa(year / 100, ovu_Gdatelbl[0], 2);			// yy
	Ovu_itoa(year % 100 / 10, ovu_Gdatelbl[1], 1);		// y
	Ovu_itoa(year % 10, ovu_Gdatelbl[2], 1);			// y
	Ovu_itoa(month, ovu_Gdatelbl[3], 2);				// mm
	Ovu_itoa(day / 10, ovu_Gdatelbl[4], 1);				// d
	Ovu_itoa(day % 10, ovu_Gdatelbl[5], 1);				// d
	
	ovu_Gperiod = 28;

}

void InitGrVariables()
{
	int i;
	U8 *pstr;
	
	for (i=0; i<5;i++)
		PushButtonSelectOff(FormGetComponentPtr(PushBtn_pmode_1015+i), FALSE);
	PushButtonSelectOn(FormGetComponentPtr(PushBtn_pmode_1015 + (ovu_Gperiod - 28)), FALSE);
	
	for (i=0; i<6; i++)
	{
		GenericButtonSetLabel(FormGetComponentPtr(PushBtn_date_1021+i), ovu_Gdatelbl[i], FALSE );	// yy
		pstr = FormGetComponentPtr(PushBtn_date_1021+i)->ComponentData.ButtonData.Label;
		pstr[0] = ovu_Gdatelbl[i][0];
		pstr[1] = ovu_Gdatelbl[i][1];
		pstr[2] = ovu_Gdatelbl[i][2];
	}
}
		        
/***************************************************************
 * InitOvu - Initialize Ovu Application              *
 ***************************************************************/
Boolean InitOvu(void) 
{
	InitVariables();

 	if(!GUIInit())
 		return FALSE;
 		
  	ClearScreen(WHITE);

  	if(!FormGo(Form_Ovu_1000))
  		return FALSE;
  		
}

Boolean DeinitOvu(void) 
{
	return TRUE;
}

void OvuDateSet(int addval)
{
	U16 cur_btn;
	U8 *pvalstr;
	U8 *pchkstr;
	S8 val;
	
	cur_btn = ButtonGroupGetSelectedButtonID(1);
	pvalstr = FormGetComponentPtr(cur_btn)->ComponentData.ButtonData.Label;
	val = Ovu_atoi(pvalstr);
	val += addval;

	pvalstr[0] = pvalstr[1] = pvalstr[2] = 0;
	
	switch (cur_btn)
	{
		case PushBtn_date_1021:	// yy
			if (val < 19) val = 19;
			else if (val > 30)	val = 30;
			pvalstr[0] = val / 10 + '0';
			pvalstr[1] = val % 10 + '0';
			break;
		case PushBtn_date_1022:	// y
		case PushBtn_date_1023:	// y
			if (val < 0) val = 9;
			else if (val > 9) val = 0;
			pvalstr[0] = val + '0';
			break;
		case PushBtn_date_1024:	// mm
			if (val < 1) val = 12;
			else if (val > 12) val = 1;
			pvalstr[0] = val / 10 + '0';
			pvalstr[1] = val % 10 + '0';
			break;
		case PushBtn_date_1025:	// d
			if (val < 0) val = 3;
			else if (val > 3) val = 0;
			pvalstr[0] = val + '0';
			break;
		case PushBtn_date_1026:	// d
			pchkstr = FormGetComponentPtr(PushBtn_date_1025)->ComponentData.ButtonData.Label;
			if (val == 0 && pchkstr[0] == '0')	val += addval;
			
			if (val < 0) val = 9;
			else if (val > 9)
			{
				if (pchkstr[0] == '0')
					val = 1;
				else
					val = 0;
			}
			if (val < 0) val = 9;
			else if (val > 9) val = 0;
			pvalstr[0] = val + '0';
			break;
	}
	
	ComponentDraw(FormGetComponentPtr(cur_btn));
	
}

U16 OvuGetUsrDate()
{
	int i;
	U8 *pbtnstr;
	
	for (i=0; i<6; i++)
	{
		pbtnstr = FormGetComponentPtr(PushBtn_date_1021 + i)->ComponentData.ButtonData.Label;
		ovu_Gdatelbl[i][0] = pbtnstr[0];
		ovu_Gdatelbl[i][1] = pbtnstr[1];
		ovu_Gdatelbl[i][2] = pbtnstr[2];
	}
}

U16 OvuGetPeriod()
{
	U16 ret;
	
	ret = 28;
	switch(ButtonGroupGetSelectedButtonID(0))
	{
		case PushBtn_pmode_1015:
			ret = 28;
			break;
		case PushBtn_pmode_1016:
			ret = 29;
			break;
		case PushBtn_pmode_1017:
			ret = 30;
			break;
		case PushBtn_pmode_1018:
			ret = 31;
			break;
		case PushBtn_pmode_1019:
			ret = 32;
			break;
	}

	return ret;
}

/****************************************************************/
/*	Handle Event : NewOvu   									*/
/****************************************************************/
Boolean NewOvuHandleEvent(EventStruct *e)
{
	Boolean handled = FALSE;

	switch(e->EventType)
	{
		case COMPONENTSELECTEVENT:
			switch(e->ObjectID)
			{
				case Button_OK_1020:
					EventPutDBoxCloseEvent();
					break;
				case RepeatBtn_date_1030:	// up
					OvuDateSet(1);
					break;
				case RepeatBtn_date_1031:	// down
					OvuDateSet(-1);
					break;
		  		default:
		  			break;
			}
			handled = TRUE;
			break;
			
		default:
			break;
	}
	return handled;
}


// Calc & Display info
void Ovulation()
{
	OVUDATE sdate, edate, odate;	// start, end, ovu date
	OVUDATE spregdate, epregdate;	// start, end pregnant date
	U16 febdays;
	U8 stry[5], strm[3], strd[3];	// string year, month, day
	
	sdate.y = Ovu_atoi(ovu_Gdatelbl[0]) * 100 + Ovu_atoi(ovu_Gdatelbl[1]) * 10 + Ovu_atoi(ovu_Gdatelbl[2]);
	sdate.m = Ovu_atoi(ovu_Gdatelbl[3]);
	sdate.d = Ovu_atoi(ovu_Gdatelbl[4]) * 10 + Ovu_atoi(ovu_Gdatelbl[5]);
	
	// check days
	febdays = 28;
	if ((sdate.y % 400) == 0 || sdate.y % 100 != 0 && sdate.y % 4 == 0) febdays = 29;
	ovu_Gdays[1] = febdays;
	if (sdate.m == 2 && sdate.d > febdays)
	{
		 sdate.d = febdays;
		 ovu_Gdatelbl[5][0] = febdays % 10 + '0';
	}
	
	DrawHorz(BLACK,  8, 78, 42, SOLID_LINE, REPLACE_STYLE);
	DrawHorz(BLACK, 50, 30, 60, SOLID_LINE, REPLACE_STYLE);
	DrawHorz(BLACK,110, 78, 42, SOLID_LINE, REPLACE_STYLE);
	
	DrawVert(BLACK, 50, 30, 48, SOLID_LINE, REPLACE_STYLE);
	DrawVert(BLACK,110, 30, 48, SOLID_LINE, REPLACE_STYLE);
	
	Ovu_addday(&sdate, &edate, ovu_Gperiod);	// month end
	Ovu_addday(&edate, &odate, -14);			// ovulation day
	Ovu_addday(&odate, &spregdate, -5);			
	Ovu_addday(&odate, &epregdate, 2);			
	
	Ovu_itoa(sdate.y, stry, 4);
	Ovu_itoa(sdate.m, strm, 2);
	Ovu_itoa(sdate.d, strd, 2);
	FMDrawString( 8,  90, stry);
	FMDrawString( 8, 102, strm);
	FMDrawString(20, 102, strd);

	Ovu_itoa(spregdate.y, stry, 4);
	Ovu_itoa(spregdate.m, strm, 2);
	Ovu_itoa(spregdate.d, strd, 2);
	FMDrawString(36,  90, stry);
	FMDrawString(36, 102, strm);
	FMDrawString(48, 102, strd);
	
	Ovu_itoa(odate.y, stry, 4);
	Ovu_itoa(odate.m, strm, 2);
	Ovu_itoa(odate.d, strd, 2);
	FMDrawString(66,  90, stry);
	FMDrawString(66, 102, strm);
	FMDrawString(78, 102, strd);

	Ovu_itoa(epregdate.y, stry, 4);
	Ovu_itoa(epregdate.m, strm, 2);
	Ovu_itoa(epregdate.d, strd, 2);
	FMDrawString( 96,  90, stry);
	FMDrawString( 96, 102, strm);
	FMDrawString(108, 102, strd);

	Ovu_itoa(edate.y, stry, 4);
	Ovu_itoa(edate.m, strm, 2);
	Ovu_itoa(edate.d, strd, 2);
	FMDrawString(126,  90, stry);
	FMDrawString(126, 102, strm);
	FMDrawString(138, 102, strd);

}

/*
	Handle Event : OvuForm
*/
Boolean OvuFormHandleEvent(EventStruct * e)
{
	Boolean handled = FALSE;

	switch (e->EventType)
	{
		case COMPONENTSELECTEVENT:
	    	if(e->ObjectID == Button_Start_1002) 
			{
				// display dialog box
    			DBoxNew(Form_NewOvu_1007);
 				InitGrVariables();
				FormSetEventHandler(NewOvuHandleEvent);
				DBoxDo(Form_NewOvu_1007);
				ovu_Gperiod = OvuGetPeriod();
				OvuGetUsrDate();
				FormFree();
				
				// calc & display info
				Ovulation();
			}

			handled = TRUE;
			break;
			
		case MENUSELECTEVENT:
			if(e->ObjectID == MenuItem_OvuAbout_1006)
				ShowAbout(GetCurAppID());
			else if (e->ObjectID == MenuItem_OvuHelp_1005)
			{
				APPHELPMSG	ahm;

				ahm.pCaptionStr = (P_U8)OvuHelpCaption;
				ahm.SbjNum = OVU_HELPSBJNUM;
				ahm.StartIdx = 0;
				ahm.pSbjStr = (P_U8 *)OvuSbjStr;
				ahm.pMsgStr = (P_U8 *)OvuMsgStr;
				
				ShowHelpMessage( (P_VOID)&ahm, APPLICATION_HELP);
			}
			
			handled = TRUE;
			break;
	}
	
 	return handled;
}

/***********************************************************************
 * ApplicationHandleEvent - Set HandelEvent to World Clock Application *
 ***********************************************************************/
Boolean ApplicationHandleEvent(EventStruct * e) 
{
	U16  id;
  	Boolean  handled = FALSE;

   
  	if(e->EventType == FORMLOADEVENT) 
	{
    	FormNew(id = e->ObjectID);

    	switch(id) 
		{
      		case Form_Ovu_1000 :           	// id 1000
      		{
      			
		        FormSetEventHandler(OvuFormHandleEvent);
        		FormDraw();
				handled = TRUE;
        		break;
        	}
    	}
  	}
  	else if(e->EventType == FORMUNLOADEVENT) 
	{
    	FormFree();
  	}

  	return handled;
}

STATUS	CommandProcessing(U32 cmd, P_U32 param)
{
	STATUS	retVal = CVOS_NOTHING;

	switch(cmd) {
		case REQ_VIEWFINDDATA:
			break;
		case REQ_DBTABLENUM:
			break;
		case REQ_DBTABLENAME:
			break;
		case REQ_DBTABLENAMEFORIR:
			break;
		case REQ_GLOBALFIND:
			break;
		case REQ_MATCHSTR:
			break;
		case REQ_UNINSTALL:
			break;
	};
	
	return retVal;
}

/***************************************************************
 * INTERNICS Main                                              *
 ***************************************************************/
STATUS AppMain(U32 cmd, P_U32 param) 
{
  	EventStruct	event;

	if(cmd != NORMAL_START)
		return ( CommandProcessing(cmd, param));
		
  	if(!InitOvu())
  	{
  		(void)MsgBoxDo( MK_ERROR, "ʱȭ ", "InitOvu()  ߻Ͽϴ.", MSGBOX_OK );
  		return CVOS_ERROR;
	}
  	
	do 
	{	/* EventLoop - Listen Event */
		while(!EventGet(&event)) /* NULL LOOP */ ;
			if(!SystemHandleEvent(&event))
				if(!MenuHandleEvent(&event))
					if(!ApplicationHandleEvent(&event))
						FormDispatchEvent(&event);
	} while(event.EventType != APPSTOPEVENT);

  	if(!DeinitOvu())
  		return CVOS_ERROR;
  	
  	return CVOS_OK;
}

